/*
 *  BattCheck
 */

#pragma pack(2)
#include <PalmOS.h>

#include "PalmApp.h"
#include "Palmrsc.h"

static char *BatKind[] = {"Alkaline", "NiCad", "LiIon", "RechAlk", "NiMH", "LiIon1400", "", "", "", "", "", ""};
static SysBatteryKind kind;
static UInt32 WriteTicks = 0;

static int StartApplication(void)
{
  FrmGotoForm(formID_AppMain);
  return 0;
}

static void StopApplication(void)
{
  FrmCloseAllForms ();
}

static void WriteBattVoltType(void)
{
  UInt16 currentVolt;
  UInt16 warnThreshold;
  UInt16 criticalThreshold;
  Int16 maxTicks;
  Boolean pluggedIn;
  UInt8 percent;
  char str[32];
  int y;
  RectangleType rt;

  WriteTicks = TimGetTicks();
  
  rt.topLeft.x = 0;
  rt.topLeft.y = 20;
  rt.extent.x = 160;
  rt.extent.y = 80;
  currentVolt = SysBatteryInfo (false, &warnThreshold, 
                &criticalThreshold, &maxTicks, &kind, &pluggedIn, &percent); 
  WinEraseRectangle(&rt, 0);
  y = 20;
  WinDrawChars ("Current Voltage", 15, 8, y);
  StrPrintF (str, "%d.%02d v (% 3d %%)", currentVolt/100, currentVolt%100, (int)percent);
  WinDrawChars (str, StrLen(str), 100, y);
  y += 16;
  
  WinDrawChars ("Warning Voltage", 15, 8, y);
  StrPrintF (str, "%d.%02d v", warnThreshold/100, warnThreshold%100);
  WinDrawChars (str, StrLen(str), 100, y);
  y += 16;
  
  WinDrawChars ("Critical Voltage", 16, 8, y);
  StrPrintF (str, "%d.%02d v", criticalThreshold/100, criticalThreshold%100);
  WinDrawChars (str, StrLen(str), 100, y);
  y += 16;
  
  WinDrawChars ("Battery kind", 12, 8, y);
  WinDrawChars (BatKind[kind], StrLen(BatKind[kind]), 100, y);
}

static void ReWriteBattVolt(void)
{
  UInt16 currentVolt;
  UInt16 warnThreshold;
  UInt16 criticalThreshold;
  Int16 maxTicks;
  Boolean pluggedIn;
  UInt8 percent;
  char str[32];
  RectangleType rt;
  
  WriteTicks = TimGetTicks();

  rt.topLeft.x = 100;
  rt.topLeft.y = 20;
  rt.extent.x = 160;
  rt.extent.y = 15;
  currentVolt = SysBatteryInfo (false, &warnThreshold, 
                &criticalThreshold, &maxTicks, &kind, &pluggedIn, &percent); 
  WinEraseRectangle(&rt, 0);
  StrPrintF (str, "%d.%02d v (% 3d %%)", currentVolt/100, currentVolt%100, (int)percent);
  WinDrawChars (str, StrLen(str), 100, 20);
}


/*
typedef enum {
  sysBatteryKindAlkaline=0,
  sysBatteryKindNiCad,
  sysBatteryKindLiIon,
  sysBatteryKindRechAlk,
  sysBatteryKindNiMH,
  sysBatteryKindLiIon1400,
  sysBatteryKindLast=0xFF   // insert new battery types BEFORE this one
  } SysBatteryKind;
*/
static void SetBattVoltType(SysBatteryKind kind)
{
  PrefSetPreference(prefSysBatteryKind, kind);
}

static void ConfReset(void)
{
  if(0 == FrmAlert(alertID_reset)){
    SysReset();
  }
}

static int ConfChange(char * str)
{
  if(0 == FrmCustomAlert(alertID_conf, str, "", "")){
    return true;
  }else{
    return false;
  }
}

static int PalmAppMenuEvt(UInt16 menuID)
{
  FormPtr   form;
  int       handled = 0;

  switch (menuID){
    case menuitemID_about:
      FrmAlert(alertID_about);
      break;
  }
  return handled;
}

static int PalmApp(EventPtr event)
{
  FormPtr   form;
  int       handled = 0;
  char str[8];

  switch (event->eType) 
  {
  case frmOpenEvent:
    form = FrmGetActiveForm();
    FrmDrawForm(form);
    WriteBattVoltType();
    handled = 1;
    break;
    
  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID== buttonID_Alk){
      if(kind == sysBatteryKindNiCad){
        if(ConfChange(BatKind[sysBatteryKindAlkaline])){
          SetBattVoltType(sysBatteryKindAlkaline);
          ConfReset();
        }
      }else if(kind != sysBatteryKindAlkaline){
        FrmAlert(alertID_stop);
      }
    }else if (event->data.ctlEnter.controlID== buttonID_Nic){
      if(kind == sysBatteryKindAlkaline){
        if(ConfChange(BatKind[sysBatteryKindNiCad])){
          SetBattVoltType(sysBatteryKindNiCad);
          ConfReset();
        }
      }else if(kind != sysBatteryKindNiCad){
        FrmAlert(alertID_stop);
      }
    }
    handled = 1;
    break;

  case menuEvent:
    handled = PalmAppMenuEvt(event->data.menu.itemID);
    break;

  case nilEvent:
    if(WriteTicks + 90 < TimGetTicks()){
      ReWriteBattVolt();
//      WriteBattVoltType();
      handled = 1;
    }
    break;
  }
  return handled;
}

static void EventLoop(void)
{
  short err;
  int formID;
  FormPtr form;
  EventType event;

  do{
    EvtGetEvent(&event, 100);  // Time Out  (Unit 0.01 sec)
    if (SysHandleEvent(&event)){
      continue;
    }
    if (MenuHandleEvent((void *)0, &event, &err)){
      continue;
    }
    if (event.eType == frmLoadEvent){
      formID = event.data.frmLoad.formID;
      form = FrmInitForm(formID);
      switch (formID) {
        case formID_AppMain:
          FrmSetEventHandler(form, (FormEventHandlerPtr) PalmApp);
          FrmSetActiveForm(form);
          break;
      }
      continue;
    }
    FrmDispatchEvent(&event);
  } while(event.eType != appStopEvent);
}

UInt32  PilotMain (UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
  int error;

  if (cmd == sysAppLaunchCmdNormalLaunch){
    error = StartApplication();
    if (error){
      return error;
    }
    EventLoop();
    StopApplication ();
  }
  return 0;
}

